
# This file is auto-generated. Do not edit!

from ovs.flow import decoders

field_decoders = {
    'dp_hash': decoders.Mask32,
    'recirc_id': decoders.decode_int,
    'packet_type': decoders.decode_default,
    'conj_id': decoders.decode_int,
    'tun_id': decoders.Mask64,
    'tunnel_id': decoders.Mask64,
    'tun_src': decoders.IPMask,
    'tun_dst': decoders.IPMask,
    'tun_ipv6_src': decoders.IPMask,
    'tun_ipv6_dst': decoders.IPMask,
    'tun_flags': decoders.decode_default,
    'tun_ttl': decoders.decode_int,
    'tun_tos': decoders.decode_int,
    'tun_gbp_id': decoders.Mask16,
    'tun_gbp_flags': decoders.Mask8,
    'tun_erspan_idx': decoders.decode_mask(20),
    'tun_erspan_ver': decoders.decode_mask(4),
    'tun_erspan_dir': decoders.decode_mask(1),
    'tun_erspan_hwid': decoders.decode_mask(6),
    'tun_gtpu_flags': decoders.Mask8,
    'tun_gtpu_msgtype': decoders.Mask8,
    'tun_metadata0': decoders.Mask992,
    'tun_metadata1': decoders.Mask992,
    'tun_metadata2': decoders.Mask992,
    'tun_metadata3': decoders.Mask992,
    'tun_metadata4': decoders.Mask992,
    'tun_metadata5': decoders.Mask992,
    'tun_metadata6': decoders.Mask992,
    'tun_metadata7': decoders.Mask992,
    'tun_metadata8': decoders.Mask992,
    'tun_metadata9': decoders.Mask992,
    'tun_metadata10': decoders.Mask992,
    'tun_metadata11': decoders.Mask992,
    'tun_metadata12': decoders.Mask992,
    'tun_metadata13': decoders.Mask992,
    'tun_metadata14': decoders.Mask992,
    'tun_metadata15': decoders.Mask992,
    'tun_metadata16': decoders.Mask992,
    'tun_metadata17': decoders.Mask992,
    'tun_metadata18': decoders.Mask992,
    'tun_metadata19': decoders.Mask992,
    'tun_metadata20': decoders.Mask992,
    'tun_metadata21': decoders.Mask992,
    'tun_metadata22': decoders.Mask992,
    'tun_metadata23': decoders.Mask992,
    'tun_metadata24': decoders.Mask992,
    'tun_metadata25': decoders.Mask992,
    'tun_metadata26': decoders.Mask992,
    'tun_metadata27': decoders.Mask992,
    'tun_metadata28': decoders.Mask992,
    'tun_metadata29': decoders.Mask992,
    'tun_metadata30': decoders.Mask992,
    'tun_metadata31': decoders.Mask992,
    'tun_metadata32': decoders.Mask992,
    'tun_metadata33': decoders.Mask992,
    'tun_metadata34': decoders.Mask992,
    'tun_metadata35': decoders.Mask992,
    'tun_metadata36': decoders.Mask992,
    'tun_metadata37': decoders.Mask992,
    'tun_metadata38': decoders.Mask992,
    'tun_metadata39': decoders.Mask992,
    'tun_metadata40': decoders.Mask992,
    'tun_metadata41': decoders.Mask992,
    'tun_metadata42': decoders.Mask992,
    'tun_metadata43': decoders.Mask992,
    'tun_metadata44': decoders.Mask992,
    'tun_metadata45': decoders.Mask992,
    'tun_metadata46': decoders.Mask992,
    'tun_metadata47': decoders.Mask992,
    'tun_metadata48': decoders.Mask992,
    'tun_metadata49': decoders.Mask992,
    'tun_metadata50': decoders.Mask992,
    'tun_metadata51': decoders.Mask992,
    'tun_metadata52': decoders.Mask992,
    'tun_metadata53': decoders.Mask992,
    'tun_metadata54': decoders.Mask992,
    'tun_metadata55': decoders.Mask992,
    'tun_metadata56': decoders.Mask992,
    'tun_metadata57': decoders.Mask992,
    'tun_metadata58': decoders.Mask992,
    'tun_metadata59': decoders.Mask992,
    'tun_metadata60': decoders.Mask992,
    'tun_metadata61': decoders.Mask992,
    'tun_metadata62': decoders.Mask992,
    'tun_metadata63': decoders.Mask992,
    'metadata': decoders.Mask64,
    'in_port': decoders.decode_default,
    'in_port_oxm': decoders.decode_default,
    'actset_output': decoders.decode_default,
    'skb_priority': decoders.decode_int,
    'pkt_mark': decoders.Mask32,
    'ct_state': decoders.decode_default,
    'ct_zone': decoders.decode_int,
    'ct_mark': decoders.Mask32,
    'ct_label': decoders.Mask128,
    'ct_nw_proto': decoders.decode_int,
    'ct_nw_src': decoders.IPMask,
    'ct_nw_dst': decoders.IPMask,
    'ct_ipv6_src': decoders.IPMask,
    'ct_ipv6_dst': decoders.IPMask,
    'ct_tp_src': decoders.Mask16,
    'ct_tp_dst': decoders.Mask16,
    'reg0': decoders.Mask32,
    'reg1': decoders.Mask32,
    'reg2': decoders.Mask32,
    'reg3': decoders.Mask32,
    'reg4': decoders.Mask32,
    'reg5': decoders.Mask32,
    'reg6': decoders.Mask32,
    'reg7': decoders.Mask32,
    'reg8': decoders.Mask32,
    'reg9': decoders.Mask32,
    'reg10': decoders.Mask32,
    'reg11': decoders.Mask32,
    'reg12': decoders.Mask32,
    'reg13': decoders.Mask32,
    'reg14': decoders.Mask32,
    'reg15': decoders.Mask32,
    'reg16': decoders.Mask32,
    'reg17': decoders.Mask32,
    'reg18': decoders.Mask32,
    'reg19': decoders.Mask32,
    'reg20': decoders.Mask32,
    'reg21': decoders.Mask32,
    'reg22': decoders.Mask32,
    'reg23': decoders.Mask32,
    'reg24': decoders.Mask32,
    'reg25': decoders.Mask32,
    'reg26': decoders.Mask32,
    'reg27': decoders.Mask32,
    'reg28': decoders.Mask32,
    'reg29': decoders.Mask32,
    'reg30': decoders.Mask32,
    'reg31': decoders.Mask32,
    'xreg0': decoders.Mask64,
    'xreg1': decoders.Mask64,
    'xreg2': decoders.Mask64,
    'xreg3': decoders.Mask64,
    'xreg4': decoders.Mask64,
    'xreg5': decoders.Mask64,
    'xreg6': decoders.Mask64,
    'xreg7': decoders.Mask64,
    'xreg8': decoders.Mask64,
    'xreg9': decoders.Mask64,
    'xreg10': decoders.Mask64,
    'xreg11': decoders.Mask64,
    'xreg12': decoders.Mask64,
    'xreg13': decoders.Mask64,
    'xreg14': decoders.Mask64,
    'xreg15': decoders.Mask64,
    'xxreg0': decoders.Mask128,
    'xxreg1': decoders.Mask128,
    'xxreg2': decoders.Mask128,
    'xxreg3': decoders.Mask128,
    'xxreg4': decoders.Mask128,
    'xxreg5': decoders.Mask128,
    'xxreg6': decoders.Mask128,
    'xxreg7': decoders.Mask128,
    'eth_src': decoders.EthMask,
    'dl_src': decoders.EthMask,
    'eth_dst': decoders.EthMask,
    'dl_dst': decoders.EthMask,
    'eth_type': decoders.decode_int,
    'dl_type': decoders.decode_int,
    'vlan_tci': decoders.Mask16,
    'dl_vlan': decoders.decode_int,
    'vlan_vid': decoders.decode_mask(12),
    'dl_vlan_pcp': decoders.decode_int,
    'vlan_pcp': decoders.decode_int,
    'mpls_label': decoders.decode_int,
    'mpls_tc': decoders.decode_int,
    'mpls_bos': decoders.decode_int,
    'mpls_ttl': decoders.decode_int,
    'ip_src': decoders.IPMask,
    'nw_src': decoders.IPMask,
    'ip_dst': decoders.IPMask,
    'nw_dst': decoders.IPMask,
    'ipv6_src': decoders.IPMask,
    'ipv6_dst': decoders.IPMask,
    'ipv6_label': decoders.decode_mask(20),
    'nw_proto': decoders.decode_int,
    'ip_proto': decoders.decode_int,
    'nw_tos': decoders.decode_int,
    'ip_dscp': decoders.decode_int,
    'nw_ecn': decoders.decode_int,
    'ip_ecn': decoders.decode_int,
    'nw_ttl': decoders.decode_int,
    'ip_frag': decoders.decode_default,
    'nw_frag': decoders.decode_default,
    'arp_op': decoders.decode_int,
    'arp_spa': decoders.IPMask,
    'arp_tpa': decoders.IPMask,
    'arp_sha': decoders.EthMask,
    'arp_tha': decoders.EthMask,
    'tcp_src': decoders.Mask16,
    'tp_src': decoders.Mask16,
    'tcp_dst': decoders.Mask16,
    'tp_dst': decoders.Mask16,
    'tcp_flags': decoders.decode_default,
    'udp_src': decoders.Mask16,
    'udp_dst': decoders.Mask16,
    'sctp_src': decoders.Mask16,
    'sctp_dst': decoders.Mask16,
    'icmp_type': decoders.decode_int,
    'icmp_code': decoders.decode_int,
    'icmpv6_type': decoders.decode_int,
    'icmpv6_code': decoders.decode_int,
    'nd_target': decoders.IPMask,
    'nd_sll': decoders.EthMask,
    'nd_tll': decoders.EthMask,
    'nd_reserved': decoders.decode_int,
    'nd_options_type': decoders.decode_int,
    'nsh_flags': decoders.Mask8,
    'nsh_mdtype': decoders.decode_int,
    'nsh_np': decoders.decode_int,
    'nsh_spi': decoders.decode_int,
    'nsp': decoders.decode_int,
    'nsh_si': decoders.decode_int,
    'nsi': decoders.decode_int,
    'nsh_c1': decoders.Mask32,
    'nshc1': decoders.Mask32,
    'nsh_c2': decoders.Mask32,
    'nshc2': decoders.Mask32,
    'nsh_c3': decoders.Mask32,
    'nshc3': decoders.Mask32,
    'nsh_c4': decoders.Mask32,
    'nshc4': decoders.Mask32,
    'nsh_ttl': decoders.decode_int,
}

field_aliases = {
    'NXOXM_ET_DP_HASH': 'dp_hash',
    'NXM_NX_DP_HASH': 'dp_hash',
    'NXM_NX_RECIRC_ID': 'recirc_id',
    'OXM_OF_PACKET_TYPE': 'packet_type',
    'NXM_NX_CONJ_ID': 'conj_id',
    'OXM_OF_TUNNEL_ID': 'tun_id',
    'NXM_NX_TUN_ID': 'tun_id',
    'NXM_NX_TUN_IPV4_SRC': 'tun_src',
    'NXM_NX_TUN_IPV4_DST': 'tun_dst',
    'NXM_NX_TUN_IPV6_SRC': 'tun_ipv6_src',
    'NXM_NX_TUN_IPV6_DST': 'tun_ipv6_dst',
    'NXM_NX_TUN_FLAGS': 'tun_flags',
    'NXM_NX_TUN_GBP_ID': 'tun_gbp_id',
    'NXM_NX_TUN_GBP_FLAGS': 'tun_gbp_flags',
    'NXOXM_ET_ERSPAN_IDX': 'tun_erspan_idx',
    'NXOXM_ET_ERSPAN_VER': 'tun_erspan_ver',
    'NXOXM_ET_ERSPAN_DIR': 'tun_erspan_dir',
    'NXOXM_ET_ERSPAN_HWID': 'tun_erspan_hwid',
    'NXOXM_ET_GTPU_FLAGS': 'tun_gtpu_flags',
    'NXOXM_ET_GTPU_MSGTYPE': 'tun_gtpu_msgtype',
    'NXM_NX_TUN_METADATA0': 'tun_metadata0',
    'NXM_NX_TUN_METADATA1': 'tun_metadata1',
    'NXM_NX_TUN_METADATA2': 'tun_metadata2',
    'NXM_NX_TUN_METADATA3': 'tun_metadata3',
    'NXM_NX_TUN_METADATA4': 'tun_metadata4',
    'NXM_NX_TUN_METADATA5': 'tun_metadata5',
    'NXM_NX_TUN_METADATA6': 'tun_metadata6',
    'NXM_NX_TUN_METADATA7': 'tun_metadata7',
    'NXM_NX_TUN_METADATA8': 'tun_metadata8',
    'NXM_NX_TUN_METADATA9': 'tun_metadata9',
    'NXM_NX_TUN_METADATA10': 'tun_metadata10',
    'NXM_NX_TUN_METADATA11': 'tun_metadata11',
    'NXM_NX_TUN_METADATA12': 'tun_metadata12',
    'NXM_NX_TUN_METADATA13': 'tun_metadata13',
    'NXM_NX_TUN_METADATA14': 'tun_metadata14',
    'NXM_NX_TUN_METADATA15': 'tun_metadata15',
    'NXM_NX_TUN_METADATA16': 'tun_metadata16',
    'NXM_NX_TUN_METADATA17': 'tun_metadata17',
    'NXM_NX_TUN_METADATA18': 'tun_metadata18',
    'NXM_NX_TUN_METADATA19': 'tun_metadata19',
    'NXM_NX_TUN_METADATA20': 'tun_metadata20',
    'NXM_NX_TUN_METADATA21': 'tun_metadata21',
    'NXM_NX_TUN_METADATA22': 'tun_metadata22',
    'NXM_NX_TUN_METADATA23': 'tun_metadata23',
    'NXM_NX_TUN_METADATA24': 'tun_metadata24',
    'NXM_NX_TUN_METADATA25': 'tun_metadata25',
    'NXM_NX_TUN_METADATA26': 'tun_metadata26',
    'NXM_NX_TUN_METADATA27': 'tun_metadata27',
    'NXM_NX_TUN_METADATA28': 'tun_metadata28',
    'NXM_NX_TUN_METADATA29': 'tun_metadata29',
    'NXM_NX_TUN_METADATA30': 'tun_metadata30',
    'NXM_NX_TUN_METADATA31': 'tun_metadata31',
    'NXM_NX_TUN_METADATA32': 'tun_metadata32',
    'NXM_NX_TUN_METADATA33': 'tun_metadata33',
    'NXM_NX_TUN_METADATA34': 'tun_metadata34',
    'NXM_NX_TUN_METADATA35': 'tun_metadata35',
    'NXM_NX_TUN_METADATA36': 'tun_metadata36',
    'NXM_NX_TUN_METADATA37': 'tun_metadata37',
    'NXM_NX_TUN_METADATA38': 'tun_metadata38',
    'NXM_NX_TUN_METADATA39': 'tun_metadata39',
    'NXM_NX_TUN_METADATA40': 'tun_metadata40',
    'NXM_NX_TUN_METADATA41': 'tun_metadata41',
    'NXM_NX_TUN_METADATA42': 'tun_metadata42',
    'NXM_NX_TUN_METADATA43': 'tun_metadata43',
    'NXM_NX_TUN_METADATA44': 'tun_metadata44',
    'NXM_NX_TUN_METADATA45': 'tun_metadata45',
    'NXM_NX_TUN_METADATA46': 'tun_metadata46',
    'NXM_NX_TUN_METADATA47': 'tun_metadata47',
    'NXM_NX_TUN_METADATA48': 'tun_metadata48',
    'NXM_NX_TUN_METADATA49': 'tun_metadata49',
    'NXM_NX_TUN_METADATA50': 'tun_metadata50',
    'NXM_NX_TUN_METADATA51': 'tun_metadata51',
    'NXM_NX_TUN_METADATA52': 'tun_metadata52',
    'NXM_NX_TUN_METADATA53': 'tun_metadata53',
    'NXM_NX_TUN_METADATA54': 'tun_metadata54',
    'NXM_NX_TUN_METADATA55': 'tun_metadata55',
    'NXM_NX_TUN_METADATA56': 'tun_metadata56',
    'NXM_NX_TUN_METADATA57': 'tun_metadata57',
    'NXM_NX_TUN_METADATA58': 'tun_metadata58',
    'NXM_NX_TUN_METADATA59': 'tun_metadata59',
    'NXM_NX_TUN_METADATA60': 'tun_metadata60',
    'NXM_NX_TUN_METADATA61': 'tun_metadata61',
    'NXM_NX_TUN_METADATA62': 'tun_metadata62',
    'NXM_NX_TUN_METADATA63': 'tun_metadata63',
    'OXM_OF_METADATA': 'metadata',
    'NXM_OF_IN_PORT': 'in_port',
    'OXM_OF_IN_PORT': 'in_port_oxm',
    'ONFOXM_ET_ACTSET_OUTPUT': 'actset_output',
    'OXM_OF_ACTSET_OUTPUT': 'actset_output',
    'NXM_NX_PKT_MARK': 'pkt_mark',
    'NXM_NX_CT_STATE': 'ct_state',
    'NXM_NX_CT_ZONE': 'ct_zone',
    'NXM_NX_CT_MARK': 'ct_mark',
    'NXM_NX_CT_LABEL': 'ct_label',
    'NXM_NX_CT_NW_PROTO': 'ct_nw_proto',
    'NXM_NX_CT_NW_SRC': 'ct_nw_src',
    'NXM_NX_CT_NW_DST': 'ct_nw_dst',
    'NXM_NX_CT_IPV6_SRC': 'ct_ipv6_src',
    'NXM_NX_CT_IPV6_DST': 'ct_ipv6_dst',
    'NXM_NX_CT_TP_SRC': 'ct_tp_src',
    'NXM_NX_CT_TP_DST': 'ct_tp_dst',
    'NXM_NX_REG0': 'reg0',
    'NXM_NX_REG1': 'reg1',
    'NXM_NX_REG2': 'reg2',
    'NXM_NX_REG3': 'reg3',
    'NXM_NX_REG4': 'reg4',
    'NXM_NX_REG5': 'reg5',
    'NXM_NX_REG6': 'reg6',
    'NXM_NX_REG7': 'reg7',
    'NXM_NX_REG8': 'reg8',
    'NXM_NX_REG9': 'reg9',
    'NXM_NX_REG10': 'reg10',
    'NXM_NX_REG11': 'reg11',
    'NXM_NX_REG12': 'reg12',
    'NXM_NX_REG13': 'reg13',
    'NXM_NX_REG14': 'reg14',
    'NXM_NX_REG15': 'reg15',
    'NXOXM_ET_REG16': 'reg16',
    'NXOXM_ET_REG17': 'reg17',
    'NXOXM_ET_REG18': 'reg18',
    'NXOXM_ET_REG19': 'reg19',
    'NXOXM_ET_REG20': 'reg20',
    'NXOXM_ET_REG21': 'reg21',
    'NXOXM_ET_REG22': 'reg22',
    'NXOXM_ET_REG23': 'reg23',
    'NXOXM_ET_REG24': 'reg24',
    'NXOXM_ET_REG25': 'reg25',
    'NXOXM_ET_REG26': 'reg26',
    'NXOXM_ET_REG27': 'reg27',
    'NXOXM_ET_REG28': 'reg28',
    'NXOXM_ET_REG29': 'reg29',
    'NXOXM_ET_REG30': 'reg30',
    'NXOXM_ET_REG31': 'reg31',
    'OXM_OF_PKT_REG0': 'xreg0',
    'OXM_OF_PKT_REG1': 'xreg1',
    'OXM_OF_PKT_REG2': 'xreg2',
    'OXM_OF_PKT_REG3': 'xreg3',
    'OXM_OF_PKT_REG4': 'xreg4',
    'OXM_OF_PKT_REG5': 'xreg5',
    'OXM_OF_PKT_REG6': 'xreg6',
    'OXM_OF_PKT_REG7': 'xreg7',
    'OXM_OF_PKT_REG8': 'xreg8',
    'OXM_OF_PKT_REG9': 'xreg9',
    'OXM_OF_PKT_REG10': 'xreg10',
    'OXM_OF_PKT_REG11': 'xreg11',
    'OXM_OF_PKT_REG12': 'xreg12',
    'OXM_OF_PKT_REG13': 'xreg13',
    'OXM_OF_PKT_REG14': 'xreg14',
    'OXM_OF_PKT_REG15': 'xreg15',
    'NXM_NX_XXREG0': 'xxreg0',
    'NXM_NX_XXREG1': 'xxreg1',
    'NXM_NX_XXREG2': 'xxreg2',
    'NXM_NX_XXREG3': 'xxreg3',
    'NXOXM_ET_XXREG4': 'xxreg4',
    'NXOXM_ET_XXREG5': 'xxreg5',
    'NXOXM_ET_XXREG6': 'xxreg6',
    'NXOXM_ET_XXREG7': 'xxreg7',
    'OXM_OF_ETH_SRC': 'eth_src',
    'NXM_OF_ETH_SRC': 'eth_src',
    'OXM_OF_ETH_DST': 'eth_dst',
    'NXM_OF_ETH_DST': 'eth_dst',
    'OXM_OF_ETH_TYPE': 'eth_type',
    'NXM_OF_ETH_TYPE': 'eth_type',
    'NXM_OF_VLAN_TCI': 'vlan_tci',
    'OXM_OF_VLAN_VID': 'vlan_vid',
    'OXM_OF_VLAN_PCP': 'vlan_pcp',
    'OXM_OF_MPLS_LABEL': 'mpls_label',
    'OXM_OF_MPLS_TC': 'mpls_tc',
    'OXM_OF_MPLS_BOS': 'mpls_bos',
    'NXM_NX_MPLS_TTL': 'mpls_ttl',
    'OXM_OF_IPV4_SRC': 'ip_src',
    'NXM_OF_IP_SRC': 'ip_src',
    'OXM_OF_IPV4_DST': 'ip_dst',
    'NXM_OF_IP_DST': 'ip_dst',
    'OXM_OF_IPV6_SRC': 'ipv6_src',
    'NXM_NX_IPV6_SRC': 'ipv6_src',
    'OXM_OF_IPV6_DST': 'ipv6_dst',
    'NXM_NX_IPV6_DST': 'ipv6_dst',
    'OXM_OF_IPV6_FLABEL': 'ipv6_label',
    'NXM_NX_IPV6_LABEL': 'ipv6_label',
    'OXM_OF_IP_PROTO': 'nw_proto',
    'NXM_OF_IP_PROTO': 'nw_proto',
    'NXM_OF_IP_TOS': 'nw_tos',
    'OXM_OF_IP_DSCP': 'ip_dscp',
    'OXM_OF_IP_ECN': 'nw_ecn',
    'NXM_NX_IP_ECN': 'nw_ecn',
    'NXM_NX_IP_TTL': 'nw_ttl',
    'NXM_NX_IP_FRAG': 'ip_frag',
    'OXM_OF_ARP_OP': 'arp_op',
    'NXM_OF_ARP_OP': 'arp_op',
    'OXM_OF_ARP_SPA': 'arp_spa',
    'NXM_OF_ARP_SPA': 'arp_spa',
    'OXM_OF_ARP_TPA': 'arp_tpa',
    'NXM_OF_ARP_TPA': 'arp_tpa',
    'OXM_OF_ARP_SHA': 'arp_sha',
    'NXM_NX_ARP_SHA': 'arp_sha',
    'OXM_OF_ARP_THA': 'arp_tha',
    'NXM_NX_ARP_THA': 'arp_tha',
    'OXM_OF_TCP_SRC': 'tcp_src',
    'NXM_OF_TCP_SRC': 'tcp_src',
    'OXM_OF_TCP_DST': 'tcp_dst',
    'NXM_OF_TCP_DST': 'tcp_dst',
    'ONFOXM_ET_TCP_FLAGS': 'tcp_flags',
    'OXM_OF_TCP_FLAGS': 'tcp_flags',
    'NXM_NX_TCP_FLAGS': 'tcp_flags',
    'OXM_OF_UDP_SRC': 'udp_src',
    'NXM_OF_UDP_SRC': 'udp_src',
    'OXM_OF_UDP_DST': 'udp_dst',
    'NXM_OF_UDP_DST': 'udp_dst',
    'OXM_OF_SCTP_SRC': 'sctp_src',
    'OXM_OF_SCTP_DST': 'sctp_dst',
    'OXM_OF_ICMPV4_TYPE': 'icmp_type',
    'NXM_OF_ICMP_TYPE': 'icmp_type',
    'OXM_OF_ICMPV4_CODE': 'icmp_code',
    'NXM_OF_ICMP_CODE': 'icmp_code',
    'OXM_OF_ICMPV6_TYPE': 'icmpv6_type',
    'NXM_NX_ICMPV6_TYPE': 'icmpv6_type',
    'OXM_OF_ICMPV6_CODE': 'icmpv6_code',
    'NXM_NX_ICMPV6_CODE': 'icmpv6_code',
    'OXM_OF_IPV6_ND_TARGET': 'nd_target',
    'NXM_NX_ND_TARGET': 'nd_target',
    'OXM_OF_IPV6_ND_SLL': 'nd_sll',
    'NXM_NX_ND_SLL': 'nd_sll',
    'OXM_OF_IPV6_ND_TLL': 'nd_tll',
    'NXM_NX_ND_TLL': 'nd_tll',
    'ERICOXM_OF_ICMPV6_ND_RESERVED': 'nd_reserved',
    'ERICOXM_OF_ICMPV6_ND_OPTIONS_TYPE': 'nd_options_type',
    'NXOXM_NSH_FLAGS': 'nsh_flags',
    'NXOXM_NSH_MDTYPE': 'nsh_mdtype',
    'NXOXM_NSH_NP': 'nsh_np',
    'NXOXM_NSH_SPI': 'nsh_spi',
    'NXOXM_NSH_SI': 'nsh_si',
    'NXOXM_NSH_C1': 'nsh_c1',
    'NXOXM_NSH_C2': 'nsh_c2',
    'NXOXM_NSH_C3': 'nsh_c3',
    'NXOXM_NSH_C4': 'nsh_c4',
    'NXOXM_NSH_TTL': 'nsh_ttl',
}
